#include "A7100Analyzer.h"
#include "A7100AnalyzerSettings.h"
#include <AnalyzerChannelData.h>


A7100Analyzer::A7100Analyzer()
    : Analyzer(),
      mSettings(new A7100AnalyzerSettings()),
      mSimulationInitilized(false)
{
    SetAnalyzerSettings(mSettings.get());
}

A7100Analyzer::~A7100Analyzer()
{
    mSettings->screenVisible = false;
    KillThread();
}


void A7100Analyzer::SetupResults()
{
}

void A7100Analyzer::WorkerThread()
{
    AnalyzerChannelData *bsyn = GetAnalyzerChannelData(mSettings->mInputChannel1);
    AnalyzerChannelData *dat1 = GetAnalyzerChannelData(mSettings->mInputChannel2);
    AnalyzerChannelData *dat2 = GetAnalyzerChannelData(mSettings->mInputChannel3);

    U64 sampleRate = GetSampleRate();

    if (bsyn->GetBitState() == BIT_LOW)
    {
        bsyn->AdvanceToNextEdge();
    }

    int line = 0;

    for (;;)
    {
        bsyn->AdvanceToNextEdge(); // start impuls
        U64 startSample = bsyn->GetSampleNumber();

        bsyn->AdvanceToNextEdge(); // ende impuls
        U64 endSample = bsyn->GetSampleNumber()+(sampleRate/(mSettings->mBitRate*4));
        U64 lineStart = 0;

        if ((((endSample-startSample)*100000000)/sampleRate) > 500)
        {
            line = 0;
        }
        else
        {
            line++;
            if (line > mSettings->screenHeight) line = mSettings->screenHeight;
        }

        for (int a=0;a<mSettings->screenWidth;a++)
        {
            U64 sample = endSample+((a*sampleRate)/mSettings->mBitRate);
            dat1->AdvanceToAbsPosition(sample);
            dat2->AdvanceToAbsPosition(sample);
            int offset = (line*mSettings->screenWidth + a);
            if (dat1->GetBitState())
            {
                if (dat2->GetBitState())
                {
                    mSettings->screenMap[offset] = 0xff00;
                }
                else
                {
                    mSettings->screenMap[offset] = 0xbf00;
                }
            }
            else
            {
                if (dat2->GetBitState())
                {
                    mSettings->screenMap[offset] = 0x6f00;
                }
                else
                {
                    mSettings->screenMap[offset] = 0;
                }
            }
        }
    }
}

bool A7100Analyzer::NeedsRerun()
{
    mSettings->calcDone = true;
    return false;
}

U32 A7100Analyzer::GenerateSimulationData(U64 minimum_sample_index, U32 device_sample_rate, SimulationChannelDescriptor **simulation_channels)
{
    if (mSimulationInitilized == false) {
        mSimulationDataGenerator.Initialize(GetSimulationSampleRate(), mSettings.get());
        mSimulationInitilized = true;
    }

    return mSimulationDataGenerator.GenerateSimulationData(minimum_sample_index, device_sample_rate, simulation_channels);
}

U32 A7100Analyzer::GetMinimumSampleRateHz()
{
    return mSettings->mBitRate * 4;
}

const char *A7100Analyzer::GetAnalyzerName() const
{
    return "A7100 Screen capture";
}

const char *GetAnalyzerName()
{
    return "A7100 Screen capture";
}

Analyzer *CreateAnalyzer()
{
    return new A7100Analyzer();
}

void DestroyAnalyzer(Analyzer *analyzer)
{
    delete analyzer;
}

