#include "A7100SimulationDataGenerator.h"
#include "A7100AnalyzerSettings.h"

A7100SimulationDataGenerator::A7100SimulationDataGenerator()
{
}

A7100SimulationDataGenerator::~A7100SimulationDataGenerator()
{
}

void A7100SimulationDataGenerator::Initialize(U32 simulation_sample_rate, A7100AnalyzerSettings *settings)
{
    mSimulationSampleRateHz = simulation_sample_rate;
    mSettings = settings;

    mClockGenerator.Init(mSettings->mBitRate, simulation_sample_rate);
    mSyn = mA7100SimulationData.Add(mSettings->mInputChannel1, mSimulationSampleRateHz, BIT_HIGH);
    mDat1 = mA7100SimulationData.Add(mSettings->mInputChannel2, mSimulationSampleRateHz, BIT_LOW);
    mDat2 = mA7100SimulationData.Add(mSettings->mInputChannel3, mSimulationSampleRateHz, BIT_LOW);
    mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByHalfPeriod(1.0));
}

U32 A7100SimulationDataGenerator::GenerateSimulationData(U64 largest_sample_requested, U32 sample_rate, SimulationChannelDescriptor **simulation_channels)
{
    U64 adjusted_largest_sample_requested = AnalyzerHelpers::AdjustSimulationTargetSample(largest_sample_requested, sample_rate, mSimulationSampleRateHz);
    while (mSyn->GetCurrentSampleNumber() < adjusted_largest_sample_requested)
    {
        if (mLine > 400)
        {
            for (int a=0; a<4; a++)
            {
                mSyn->Transition();
                mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByTimeS(0.000046));
                mSyn->Transition();
                mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByTimeS(0.000002));
            }
            mSyn->Transition();
            mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByTimeS(0.000002));
            mSyn->Transition();
            mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByTimeS(0.000002));
            mColum = 0;
            mLine = 0;
        }

        if (mColum > 800)
        {
            if (mLine < 400)
            {
                mSyn->Transition();
                mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByTimeS(0.000002));
                mSyn->Transition();
                mColum = 0;
            }
            mLine++;
        }

        if (mLine > 4 && mLine < 398 && mColum > 4 && mColum < 700)
        {
            if (!((mColum^mLine) & 9))
            {
                mDat1->Transition();
            }
            if (!((mColum^mLine) & 13))
            {
                mDat2->Transition();
            }
        }
        else
        {
            mDat1->TransitionIfNeeded(BIT_LOW);
            mDat2->TransitionIfNeeded(BIT_LOW);
        }
        mA7100SimulationData.AdvanceAll(mClockGenerator.AdvanceByHalfPeriod());
        mColum++;
    }

    *simulation_channels = mA7100SimulationData.GetArray();
    return 3;
}

